package com.dvp.tutoriel.fibonacci;

import static junit.framework.Assert.assertEquals;

import org.junit.Before;
import org.junit.Test;

/**
 * Tests de la calculette "MaCalculette.java"
 * 
 * @author Thierry Leriche-Dessirier
 *
 */
public class MaCalculetteTest {

	private Calculette calculette;

	@Before
	public void doBefore() {
		calculette = new MaCalculette();
	}

	/**
	 * Test du calcul de la Suite de Fibonacci.
	 * 
	 * REGLE RG024.1 : f(1) = 1
	 * 
	 * PARAM n = 1 
	 * RESULT = 1
	 */
	@Test
	public void testFibonacci_RG024_1() {
		// Arrange
		final Integer n = 1;
		final long resultatAttendu = 1;

		// Act and assert
		testFibonacci_RG024_x(n, resultatAttendu);
	}
	
	/**
	 * Test du calcul de la Suite de Fibonacci.
	 * 
	 * REGLE RG024.2 : f(2) = 1
	 * 
	 * PARAM n = 2 
	 * RESULT = 1
	 */
	@Test
	public void testFibonacci_RG024_2() {
		// Arrange
		final Integer n = 2;
		final long resultatAttendu = 1;

		// Act and assert
		testFibonacci_RG024_x(n, resultatAttendu);
	}
	
	/**
	 * Test du calcul de la Suite de Fibonacci.
	 * 
	 * REGLE RG024.3.a : f(3) = 2
	 * 
	 * PARAM n = 3 
	 * RESULT = 2
	 */
	@Test
	public void testFibonacci_RG024_3_a() {
		// Arrange
		final Integer n = 3;
		final long resultatAttendu = 2;

		// Act and assert
		testFibonacci_RG024_x(n, resultatAttendu);
	}
	
	/**
	 * Test du calcul de la Suite de Fibonacci.
	 * 
	 * REGLE RG024.3.b : f(4) = 3
	 * 
	 * PARAM n = 4 
	 * RESULT = 3
	 */
	@Test
	public void testFibonacci_RG024_3_b() {
		// Arrange
		final Integer n = 4;
		final long resultatAttendu = 3;

		// Act and assert
		testFibonacci_RG024_x(n, resultatAttendu);
	}
	
	/**
	 * Test du calcul de la Suite de Fibonacci.
	 * 
	 * REGLE RG024.3.c : f(5) = 5
	 * 
	 * PARAM n = 5 
	 * RESULT = 5
	 */
	@Test
	public void testFibonacci_RG024_3_c() {
		// Arrange
		final Integer n = 5;
		final long resultatAttendu = 5;

		// Act and assert
		testFibonacci_RG024_x(n, resultatAttendu);
	}
	
	/**
	 * Test du calcul de la Suite de Fibonacci.
	 * 
	 * REGLE RG024.3.d : f(6) = 8
	 * 
	 * PARAM n = 6 
	 * RESULT = 8
	 */
	@Test
	public void testFibonacci_RG024_3_d() {
		// Arrange
		final Integer n = 6;
		final long resultatAttendu = 8;

		// Act and assert
		testFibonacci_RG024_x(n, resultatAttendu);
	}
	
	/**
	 * Test du calcul de la Suite de Fibonacci.
	 * 
	 * REGLE RG024.3.e : f(7) = 13
	 * 
	 * PARAM n = 7 
	 * RESULT = 13
	 */
	@Test
	public void testFibonacci_RG024_3_e() {
		// Arrange
		final Integer n = 7;
		final long resultatAttendu = 13;

		// Act and assert
		testFibonacci_RG024_x(n, resultatAttendu);
	}
	
	/**
	 * Test du calcul de la Suite de Fibonacci.
	 * 
	 * REGLE RG024.3.f : f(8) = 21
	 * 
	 * PARAM n = 8 
	 * RESULT = 21
	 */
	@Test
	public void testFibonacci_RG024_3_f() {
		// Arrange
		final Integer n = 8;
		final long resultatAttendu = 21;

		// Act and assert
		testFibonacci_RG024_x(n, resultatAttendu);
	}
	
	/**
	 * Test du calcul de la Suite de Fibonacci.
	 * 
	 * REGLE RG024.4.a : Il n'est pas possible de calculer la valeur de la Suite
	 * de Fibonacci pour un rang negatif ou nul. 
	 * 
	 * PARAM n = 0 
	 * RESULT = Exception
	 */
	@Test(expected = IllegalArgumentException.class)
	public void testFibonacci_RG024_4_a() {
		// Arrange
		final Integer n = 0;

		// Act and assert
		testFibonacci_RG024_x(n, null);
	}
	
	/**
	 * Test du calcul de la Suite de Fibonacci.
	 * 
	 * REGLE RG024.4.b : Il n'est pas possible de calculer la valeur de la Suite
	 * de Fibonacci pour un rang negatif ou nul. 
	 * 
	 * PARAM n = -1 
	 * RESULT = Exception
	 */
	@Test(expected = IllegalArgumentException.class)
	public void testFibonacci_RG024_4_b() {
		// Arrange
		final Integer n = -1;

		// Act and assert
		testFibonacci_RG024_x(n, null);
	}
	
	/**
	 * Test du calcul de la Suite de Fibonacci.
	 * 
	 * REGLE RG024.5 : f(55) = 139583862445
	 * 
	 * PARAM n = 55 
	 * RESULT = 225851433717 (en moins de 2 secondes)
	 */
	@Test(timeout = 2000)
	public void testFibonacci_RG024_5() {
		// Arrange
		final Integer n = 55;
		final long resultatAttendu = 139583862445L;

		// Act and assert
		testFibonacci_RG024_x(n, resultatAttendu);
	}
	
	/**
	 * Test du calcul de la Suite de Fibonacci.
	 * 
	 * REGLE RG024.6 : f(49) = 7778742049
	 * 
	 * PARAM n = 49 
	 * RESULT = 7778742049 (en moins de 1 seconde)
	 */
	@Test(timeout = 1000)
	public void testFibonacci_RG024_6() {
		// Arrange
		final Integer n = 49;
		final long resultatAttendu = 7778742049L;

		// Act and assert
		testFibonacci_RG024_x(n, resultatAttendu);
	}



	private void testFibonacci_RG024_x(Integer n, Long resultatAttendu) {
		// Act
		final Long resultatCalcule = calculette.fibonacci(n);

		// Assert
		assertEquals(resultatAttendu, resultatCalcule);
	}

}
